//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Ms3ViewerUnit.h"
#include "Ms3FrameListUnit.h"
#include "DataFrame.h"
#include "display\ValueStringFormatter.h"
#include "display\Display.h"
#include "device\MiniscopeDeviceInterface.h"
#include "Ms3Recorder.h"
#include "Log.h"

#include <fstream>
#include <time.h>
#include <stdio.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMs3Viewer *frmMs3Viewer;
//---------------------------------------------------------------------------
__fastcall TfrmMs3Viewer::TfrmMs3Viewer(TComponent* Owner)
	: TForm(Owner),
	callbackClose(NULL),
	data(NULL),
	frame_cnt(0),
	current_frame(0)
{
}
//---------------------------------------------------------------------------

void __fastcall TfrmMs3Viewer::FormClose(TObject *Sender, TCloseAction &Action)
{
	frmMs3FrameList->Close();
	if (callbackClose)
	{
		callbackClose();
	}
}
//---------------------------------------------------------------------------

void __fastcall TfrmMs3Viewer::btnExitViewerClick(TObject *Sender)
{
	Close();	
}
//---------------------------------------------------------------------------

int TfrmMs3Viewer::LoadFile(AnsiString asFileName, Display *display,
	std::vector<float> *vdata)
{
	data = vdata;
	int status = Ms3Recorder::Instance().SetMode(Ms3Recorder::MODE_READ, asFileName);
	if (status)
		return status;
	asCurrentFile = asFileName;
	this->display = display;
	current_frame = 0;
	frame_cnt = Ms3Recorder::Instance().GetFrameCount();
	frmMs3FrameList->UpdateView();
	data->clear();
	if (frame_cnt > 0)
	{
		status = LoadFrame();
	}
	else
	{
		display->Update(Display::UPDATE_ALL);
		lblFrameVal->Caption = "---";
	}

	return status;
}

int TfrmMs3Viewer::LoadFrame(void)
{
	struct S_DATAFRAME desc;
	int status = Ms3Recorder::Instance().GetFrame(current_frame, desc, data);
	if (status)
	{
		LOG(E_LOG_TRACE, "Failed to get data frame (status = %d)", status);
		return status;
	}
	else
	{
		edFilename->Text = asCurrentFile;
		float fSamplVal = (float)1.0 / desc.fDivX;
		lblSamplingVal->Caption =
			ValueStringFormatter::FreqToStr(fSamplVal, 1).c_str();

		lblTriggerTypeVal->Caption = DeviceInterface::GetTriggerTypeName((E_TRIGGER_TYPE)desc.trigger_type);
		lblCouplingTypeVal->Caption = DeviceInterface::GetCouplingTypeName((E_COUPLING_TYPE)desc.coupling);
		char* timeline = ctime( & ( desc.timestamp.time ) );
		timeline[19]=0;
		char buf[64];
		sprintf(buf, "%15s.%03hu", &timeline[4], desc.timestamp.millitm);
		lblTimestampVal->Caption = buf;
		lblSamplesCountVal->Caption = desc.buffer_size;

		display->SetDiv(desc.fDivX);
		display->Update(Display::UPDATE_ALL);

		AnsiString asFrameVal;
		asFrameVal.sprintf("%d of %d", current_frame + 1, frame_cnt);
		lblFrameVal->Caption = asFrameVal;
	}
	return status;
}


void __fastcall TfrmMs3Viewer::btnPrevFrameClick(TObject *Sender)
{
	if (frame_cnt == 0 || current_frame == 0)
		return;
	current_frame--;
	frmMs3FrameList->UpdateView();
	int status = LoadFrame();
	if (status)
		LOG(E_LOG_TRACE, "Ms3Viewer: failed to load frame %d with status %d", current_frame, status);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMs3Viewer::btnNextFrameClick(TObject *Sender)
{
	if (frame_cnt == 0 || current_frame >= frame_cnt-1)
		return;
	current_frame++;
	frmMs3FrameList->UpdateView();
	int status = LoadFrame();
	if (status)
		LOG(E_LOG_TRACE, "Ms3Viewer: failed to load frame %d with status %d", current_frame, status);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMs3Viewer::btnFrameListClick(TObject *Sender)
{
	frmMs3FrameList->Show();	
}
//---------------------------------------------------------------------------

